/* K & R C version */
#include <math.h>
#include "LIB/message.h"

float sph_spline(Delta,Delta_bar) 
float Delta,Delta_bar;
{
/* Wang & Dahlen GRL Vol 22 No 22 pp 3099-3102 Nov 15 95 */
float f_D;
  
  if(Delta<=Delta_bar) {
    f_D =3./4.*(1./(Delta_bar*Delta_bar*Delta_bar))*Delta*Delta*Delta;
    f_D-=6./4.*(1./(Delta_bar*Delta_bar))*Delta*Delta;
    f_D+=1.;
  }
  else if(Delta<=2.*Delta_bar) {
  float Delta_1,Delta_1_bar;
    Delta_1=Delta-Delta_bar;
    Delta_1_bar=Delta_1/Delta_bar;
    f_D =-Delta_1_bar*Delta_1_bar*Delta_1_bar;
    f_D+=3*Delta_1_bar*Delta_1_bar;
    f_D-=3*Delta_1_bar;
    f_D+=1.;
    f_D/=4.;
  }
  else f_D=0.0;
  return(f_D);
}

float cub_bspline(ord,nknots,knot,h,xi) 
int ord,nknots;
float *knot,*h,xi;
{
/* Michelini & McEvilly BSSA Vol 81 p 529, April 1991 (different normalisation) */
/* ord 	: ord of rho_x
   nknots=Nx + 1 : number of knots (Nx=index of highest spline)
   h[i] 	: spacing of segment between i&i+1
   knot[] 	: value of knot
   xi		: knot of interest
 */
int ii,Nx;
float rho_x;
  Nx=nknots-1;
  if(ord==0) {
    if(xi>=knot[ord]&&xi<=knot[ord+1]) {
      rho_x =1./(6.*h[ord]*h[ord]*h[ord])*(xi-knot[ord])*(xi-knot[ord])*(xi-knot[ord]);
      rho_x-=1./(h[ord])*(xi-knot[ord]);
      rho_x+=1.;
    }
    else if(xi>=knot[ord+1]&&xi<=knot[ord+2]) {
      rho_x =-1./(6.*h[ord+1]*h[ord+1]*h[ord+1])*(xi-knot[ord+1])*(xi-knot[ord+1])*(xi-knot[ord+1]);
      rho_x+=1./(2.*h[ord+1]*h[ord+1])*(xi-knot[ord+1])*(xi-knot[ord+1]);
      rho_x-=1./(2.*h[ord+1])*(xi-knot[ord+1]);
      rho_x+=1./6.;
    }
    else rho_x=0.0;
  }
  
  else if(ord==1) {
    if(xi>=knot[ord-1]&&xi<=knot[ord]) {
      rho_x =-1./(3.*h[ord-1]*h[ord-1]*h[ord-1])*(xi-knot[ord-1])*(xi-knot[ord-1])*(xi-knot[ord-1]);
      rho_x+=1./(h[ord-1])*(xi-knot[ord-1]);
    }
    else if(xi>=knot[ord]&&xi<=knot[ord+1]) {
      rho_x =1./(2.*h[ord]*h[ord]*h[ord])*(xi-knot[ord])*(xi-knot[ord])*(xi-knot[ord]);
      rho_x-=1./(h[ord]*h[ord])*(xi-knot[ord])*(xi-knot[ord]);
      rho_x+=2./3.;
    }
    else if(xi>=knot[ord+1]&&xi<=knot[ord+2]) {
      rho_x =-1./(6.*h[ord+1]*h[ord+1]*h[ord+1])*(xi-knot[ord+1])*(xi-knot[ord+1])*(xi-knot[ord+1]);
      rho_x+=1./(2.*h[ord+1]*h[ord+1])*(xi-knot[ord+1])*(xi-knot[ord+1]);
      rho_x-=1./(2.*h[ord+1])*(xi-knot[ord+1]);
      rho_x+=1/6.;
    }
    else rho_x=0.0;
  }
  
  else if(ord==Nx-1) {
    if(xi<=knot[ord-2]) rho_x=0.0;
    else if(xi>=knot[ord-2]&&xi<=knot[ord-1])
      rho_x =1./(6.*h[ord-2]*h[ord-2]*h[ord-2])*(xi-knot[ord-2])*(xi-knot[ord-2])*(xi-knot[ord-2]);
    else if(xi>=knot[ord-1]&&xi<=knot[ord]) {
      rho_x =-1./(2.*h[ord-1]*h[ord-1]*h[ord-1])*(xi-knot[ord-1])*(xi-knot[ord-1])*(xi-knot[ord-1]);
      rho_x+=1./(2.*h[ord-1]*h[ord-1])*(xi-knot[ord-1])*(xi-knot[ord-1]);
      rho_x+=1./(2.*h[ord-1])*(xi-knot[ord-1]);
      rho_x+=1/6.;
    }
    else if(xi>=knot[ord]&&xi<=knot[ord+1]) {
      rho_x =1./(3.*h[ord]*h[ord]*h[ord])*(xi-knot[ord])*(xi-knot[ord])*(xi-knot[ord]);
      rho_x-=1./(h[ord]*h[ord])*(xi-knot[ord])*(xi-knot[ord]);
      rho_x+=2./3.;
    }
  }
  
  else if(ord==Nx) {
    if(xi<=knot[ord-2]) rho_x=0.0;
    else if(xi>=knot[ord-2]&&xi<=knot[ord-1])
      rho_x =1./(6.*h[ord-2]*h[ord-2]*h[ord-2])*(xi-knot[ord-2])*(xi-knot[ord-2])*(xi-knot[ord-2]);
    else if(xi>=knot[ord-1]&&xi<=knot[ord]) {
      rho_x =-1./(6.*h[ord-1]*h[ord-1]*h[ord-1])*(xi-knot[ord-1])*(xi-knot[ord-1])*(xi-knot[ord-1]);
      rho_x+=1./(2.*h[ord-1]*h[ord-1])*(xi-knot[ord-1])*(xi-knot[ord-1]);
      rho_x+=1./(2.*h[ord-1])*(xi-knot[ord-1]);
      rho_x+=1/6.;
    }
  }
  
  else {	/* Gal expression >2 knots away from either boundary */
    if(xi<=knot[ord-2]||xi>=knot[ord+2]) rho_x=0.0;
    
    else if(xi>=knot[ord-2]&&xi<=knot[ord-1])
      rho_x =1./(6.*h[ord-2]*h[ord-2]*h[ord-2])*(xi-knot[ord-2])*(xi-knot[ord-2])*(xi-knot[ord-2]);
      
    else if(xi>=knot[ord-1]&&xi<=knot[ord]) {
      rho_x =-1./(2.*h[ord-1]*h[ord-1]*h[ord-1])*(xi-knot[ord-1])*(xi-knot[ord-1])*(xi-knot[ord-1]);
      rho_x+=1./(2.*h[ord-1]*h[ord-1])*(xi-knot[ord-1])*(xi-knot[ord-1]);
      rho_x+=1./(2.*h[ord-1])*(xi-knot[ord-1]);
      rho_x+=1/6.;
    }
    
    else if(xi>=knot[ord]&&xi<=knot[ord+1]) {
      rho_x =1./(2.*h[ord]*h[ord]*h[ord])*(xi-knot[ord])*(xi-knot[ord])*(xi-knot[ord]);
      rho_x-=1./(h[ord]*h[ord])*(xi-knot[ord])*(xi-knot[ord]);
      rho_x+=2./3.;
    }
    
    else if(xi>=knot[ord+1]&&xi<=knot[ord+2]) {
      rho_x =-1./(6.*h[ord+1]*h[ord+1]*h[ord+1])*(xi-knot[ord+1])*(xi-knot[ord+1])*(xi-knot[ord+1]);
      rho_x+=1./(2.*h[ord+1]*h[ord+1])*(xi-knot[ord+1])*(xi-knot[ord+1]);
      rho_x-=1./(2.*h[ord+1])*(xi-knot[ord+1]);
      rho_x+=1/6.;
    }
    
  }
  if(rho_x>1.)
    STOP(spline: algorithm);
return(rho_x);
}
      
    
    

      
    

  
 
   
      
    
